function fisheyeTransform(x, focus, distortion, min, max) {
    var left = x < focus;
    var m = (left ? focus - min : max - focus) || max - min;
    var f = left ? -1 : 1;
    return (f * m * (distortion + 1)) / (distortion + m / ((x - focus) * f)) + focus;
}
function fisheyeUntransform(tx, focus, distortion, min, max) {
    var left = tx < focus;
    var m = (left ? focus - min : max - focus) || max - min;
    var f = left ? -1 : 1;
    return m / ((m * (distortion + 1)) / (tx - focus) - distortion * f) + focus;
}
/**
 * Applies cartesian fisheye transforms for the first dimension of vector2.
 * @param params [focus, distortion]
 * @param x x of the the bounding box of coordinate
 * @param y y of the the bounding box of coordinate
 * @param width width of the the bounding box of coordinate
 * @param height height of the the bounding box of coordinate
 * @returns transformer
 */
export var fisheyeX = function (params, x, y, width, height) {
    var _a = params, focus = _a[0], distortion = _a[1];
    return {
        transform: function (vector) {
            var vx = vector[0], vy = vector[1];
            var fx = fisheyeTransform(vx, focus, distortion, x, x + width);
            return [fx, vy];
        },
        untransform: function (vector) {
            var fx = vector[0], vy = vector[1];
            var vx = fisheyeUntransform(fx, focus, distortion, x, x + width);
            return [vx, vy];
        },
    };
};
/**
 * Applies cartesian fisheye transforms for the second dimension of vector2.
 * @param params [focus, distortion]
 * @param x x of the the bounding box of coordinate
 * @param y y of the the bounding box of coordinate
 * @param width width of the the bounding box of coordinate
 * @param height height of the the bounding box of coordinate
 * @returns transformer
 */
export var fisheyeY = function (params, x, y, width, height) {
    var _a = params, focus = _a[0], distortion = _a[1];
    return {
        transform: function (vector) {
            var vx = vector[0], vy = vector[1];
            var fy = fisheyeTransform(vy, focus, distortion, y, y + height);
            return [vx, fy];
        },
        untransform: function (vector) {
            var vx = vector[0], fy = vector[1];
            var vy = fisheyeUntransform(fy, focus, distortion, y, y + height);
            return [vx, vy];
        },
    };
};
/**
 * Applies cartesian fisheye transforms for both dimensions of vector2.
 * @param params [focusX, focusY, distortionX, distortionY]
 * @param x x of the the bounding box of coordinate
 * @param y y of the the bounding box of coordinate
 * @param width width of the the bounding box of coordinate
 * @param height height of the the bounding box of coordinate
 * @returns transformer
 */
export var fisheye = function (params, x, y, width, height) {
    var _a = params, focusX = _a[0], focusY = _a[1], distortionX = _a[2], distortionY = _a[3];
    return {
        transform: function (vector) {
            var vx = vector[0], vy = vector[1];
            var fx = fisheyeTransform(vx, focusX, distortionX, x, x + width);
            var fy = fisheyeTransform(vy, focusY, distortionY, y, y + height);
            return [fx, fy];
        },
        untransform: function (vector) {
            var fx = vector[0], fy = vector[1];
            var vx = fisheyeUntransform(fx, focusX, distortionX, x, x + width);
            var vy = fisheyeUntransform(fy, focusY, distortionY, y, y + height);
            return [vx, vy];
        },
    };
};
/**
 * Applies circular fisheye transforms.
 * @param params [focusX, focusY, radius, distortion]
 * @param x x of the the bounding box of coordinate
 * @param y y of the the bounding box of coordinate
 * @param width width of the the bounding box of coordinate
 * @param height height of the the bounding box of coordinate
 * @returns transformer
 */
export var fisheyeCircular = function (params, x, y, width, height) {
    var _a = params, focusX = _a[0], focusY = _a[1], radius = _a[2], distortion = _a[3];
    return {
        transform: function (vector) {
            var x = vector[0], y = vector[1];
            var dx = x - focusX;
            var dy = y - focusY;
            var dd = Math.sqrt(dx * dx + dy * dy);
            if (dd > radius)
                return [x, y];
            var r = fisheyeTransform(dd, 0, distortion, 0, radius);
            var theta = Math.atan2(dy, dx);
            return [focusX + r * Math.cos(theta), focusY + r * Math.sin(theta)];
        },
        untransform: function (vector) {
            var tx = vector[0], ty = vector[1];
            var dx = tx - focusX;
            var dy = ty - focusY;
            var dd = Math.sqrt(dx * dx + dy * dy);
            if (dd > radius)
                return [tx, ty];
            var x = fisheyeUntransform(dd, 0, distortion, 0, radius);
            var theta = Math.atan2(dy, dx);
            return [focusX + x * Math.cos(theta), focusY + x * Math.sin(theta)];
        },
    };
};
//# sourceMappingURL=fisheye.js.map